# -*- coding: utf-8 -*-
# @Author: home-pc
# @Date:   2017-09-10 15:25:52
# @Last Modified by:   hang.zhang
# @Last Modified time: 2018-03-04 14:03:31


from scrapy_redis.queue import PriorityQueue


class easyPriorityQueue(PriorityQueue):

    pass



# coding=utf-8
import traceback
import logging
from scrapy_redis import connection
from scrapy.downloadermiddlewares.retry import RetryMiddleware
"""

[当连接超时(代理无效的时候要重试)或者当500错误被屏蔽的时候要重试, 而不是直接报错误而放弃]
"""

logger = logging.getLogger(__name__)


class retryToStartMiddleware(RetryMiddleware):
    """

    [请求失败后被重置是放在队尾，也就是 rpush]
    """

    def __init__(self, settings):
        super(retryToStartMiddleware, self).__init__(settings)
        self.server = connection.from_settings(settings)

    def _retry(self, request, reason, spider):
        retries = request.meta.get('retry_times', 0) + 1

        if retries <= self.max_retry_times:
            logger.debug("Retrying %(request)s (failed %(retries)d times): %(reason)s",
                         {'request': request, 'retries': retries, 'reason': reason},
                         extra={'spider': spider})
            retryreq = request.copy()
            retryreq.meta['retry_times'] = retries
            # retryreq.dont_filter = True
            retryreq.priority = request.priority + self.priority_adjust
            return retryreq
        else:
            # print "\n\n\n\n\n\n  in else \n\n\n\n"
            logger.debug("Gave up retrying %(request)s (failed %(retries)d times): %(reason)s",
                         {'request': request, 'retries': retries, 'reason': reason},
                         extra={'spider': spider})
            # put_back_2_start_url 做了request还是response 的判断...可惜blocked_call_back没有，不然上面的logger也能合并为一条
            msg = traceback.format_exc(reason)
            # easyspider = request.meta.get("easyspider") or {}
            # if not easyspider:
            #     request.meta["easyspider"] = {"from_retry": 1}
            # else:
            #     request.meta["easyspider"]["from_retry"] += 1
            spider.put_back_2_start_url(request, exc_info={
                "request": request.url,
                "traceback": msg,
                "body": None,
            })



# -*- coding: utf-8 -*-
# @Author: hang.zhang
# @Date:   2018-03-05 19:10:07
# @Last Modified by:   hang.zhang
# @Last Modified time: 2019-03-06 15:58:14
import json
import time
import logging
import hashlib
from DBService import MysqlService
from twisted.internet.threads import deferToThread
from easyspider.pipelines.commonpipeline import commonpipeline
import copy
logger = logging.getLogger(__name__)


def md5_from_dict(item):
    """注意一个严重问题：！！
            Duplicate entry '001071' for key 'hash_check_item'")

            >>> md5_from_dict({"code": "001071"})
            '05e9ab87d2f88fbc29cf070b6241c0ea'
            >>> md5_from_dict({"code": u"001071"})
            '98f46fdaffc7e924e1c565c9f182ee6e'

    带不带u 差距巨大
    """
    sort_list = sorted(item.iteritems(), key=lambda x: x[0])
    # 全部统一成str
    sort_list = map(lambda s: map(str, s), sort_list)
    return hashlib.md5(str(sort_list)).hexdigest()

# insert_or_update 为了标志是否进行过操作，一定要加上一个时间的字段，必须要有时间


def current_time():
    return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time()))


def process_item(server, item, table=None, db=None, hash_check_item=None, unique_key="id", hash_check_enable=True):
    _save_item = item.copy()

    if not _save_item.get("save_mysql", True) or not _save_item.get("easyspider", {}).get("save_mysql", True):
        return item

    pop_key_list = ["crawled_urls_path", "spider", "crawled_server", "crawled_time"]

    for key in pop_key_list:
        if key in _save_item:
            _save_item.pop(key)

    # 用什么校验，来防止重复插入
    hash_check_item = _save_item.pop("hash_check_item", None)

    if _save_item.get("easyspider"):
        easyspider_meta = _save_item.pop("easyspider")
    else:
        easyspider_meta = {}
    mysql_config = easyspider_meta.get("mysql_config") or {}

    table = table or mysql_config.get("table")
    db = db or mysql_config.get("db")
    # 兼容以前的老版写法，以前的写法是，hash_check直接写在最外面
    # 现在是如果没有发现以前的老版本写法的话，那么就使用新版本的写法
    if not hash_check_item:
        hash_check_item = mysql_config.get("hash_check_item")
    unique_key = mysql_config.get("unique_key")
    hash_check_enable = mysql_config.get("hash_check_enable", True)
    # 多处理两种模式：
    # 1. 如果数据存在，是原地更改，还是删除，以前的操作都是原地更改
    # 2. 如果要更新，是先删除再更新，还是原地更改update

    if_exists = mysql_config.get("if_exists") or "update"
    update_type = mysql_config.get("update_type'") or "inplace"

    # 最后
    error_flag = False
    error_limit = 3
    error_count = 1
    while error_count <= error_limit:
        try:
            _insert_or_update(server, _save_item, table, db, hash_check_item, unique_key, hash_check_enable, if_exists, update_type)
            error_flag = False
            break
        except Exception:
            error_flag = True
            logger.exception(u"第[%s / %s]操作数据库失败，下次再尝试" % (error_count, error_limit))
            time.sleep(2)

        error_count += 1

    # 如果最后一次还是错了，那么再来一次，还是不行，就算了
    if error_flag:
        _insert_or_update(server, _save_item, table, db, hash_check_item, unique_key, hash_check_enable)

    return item


def _insert_or_update(server, item, table=None, db=None, hash_check_item=None, unique_key="id", hash_check_enable=True, if_exists="update", update_type="inplace"):

    # 适应可以指定表的情况
    # -------------------------------------------------------------
    if not table:
        current_table = ""
    else:
        current_table = table

    if not db:
        current_db = ""
    else:
        current_db = db

    if not hash_check_item:
        # 注意，这里没有copy 因为_hash_check_item不会去改变源数据
        _hash_check_item = item
    else:
        # 有hash_check_item，重新装载
        _hash_check_item = {}
        for check_item in hash_check_item:
            _hash_check_item[check_item] = item.get(check_item)

    # 默认以 id 作为unique key
    if not unique_key:
        unique_key = "id"

    if hash_check_enable:
        # 启用hash_check_enable的话，简单直观就是去检索 hash_code
        hash_code = md5_from_dict(_hash_check_item)
        query_map = 'hash_check="%s"' % hash_code
    else:
        # 不启用，只好一个个的 and 下去
        query_map = MysqlService.join_query_map(_hash_check_item)

    check_sql = "select %s from %s.%s where %s;" % (unique_key, current_db, current_table, query_map)
    # 最终生成的查询，如果是hash_check 形式的话，那么就是原来的
    # hashcode_list = self.server.query('select id from %s.%s where hash_check="%s";' % (current_db, current_table, hash_code))
    # 如果不是 hash_check形式的话，那么就是组合的 key=value
    check_result = server.query(check_sql)
    if check_result:
        # 找到记录，那么后续的操作就是update更新
        unique_key_item = check_result[0].get(unique_key)
        if hash_check_enable:
            item["last_checktime"] = current_time()
        update_sql = server.update_sql_from_map(
            current_table, {unique_key: unique_key_item}, item, current_db).replace("%", "%%")
        logger.debug(
            "already have record, update last_checktime, running sql is %s" % update_sql)
        server.execute(update_sql)
    else:
        # 找不到记录，那么就是直接插入
        if hash_check_enable:
            item["hash_check"] = hash_code
            item["last_checktime"] = current_time()
        sql = server.join_sql_from_map(
            current_table, item, current_db).replace("%", "%%")
        logger.debug("find a new record, insert sql is %s" % sql)
        server.execute(sql)


# class commonMysqlpipeline(object):
class commonMysqlpipeline(commonpipeline):

    def __init__(self, settings):
        self.mysql_host = settings.get("MYSQL_HOST")
        self.mysql_user = settings.get("MYSQL_USER")
        self.mysql_password = settings.get("MYSQL_PASSWORD")
        self.mysql_port = settings.get("MYSQL_PORT")
        self.mysql_db = settings.get("MYSQL_DB")
        self.mysql_table = settings.get("MYSQL_TABLE")

        self.server = MysqlService(
            self.mysql_host, self.mysql_user, self.mysql_password, self.mysql_port)
        self.server.select_db(self.mysql_db)

    @classmethod
    def from_settings(cls, settings):
        return cls(settings)

    @classmethod
    def from_crawler(cls, crawler):
        return cls.from_settings(crawler.settings)

    # def process_item(self, item, spider, response):
    #     d = deferToThread(self._process_item, item, spider, response)

    #     def error_back(err):
    #         # 既然出错，那么就要block_callback记录，重新放回起始队列
    #         r_copy = response.copy()
    #         r_copy.request.meta["easyspider"]["from_retry"] = 9999
    #         # 带上item 方便检查错误在哪
    #         msg = "Error processing %s ; %s" % (json.dumps(item), err.getTraceback())
    #         spider.put_back_2_start_url(response,
    #                                     exc_info=msg,
    #                                     )

    #     d.addErrback(error_back)
    #     return d

    def _process_item(self, item, spider, response):
        _save_item = item.copy()
        # 兼容以前老的写法，老的写法是，save_mysql 直接写在最外层
        # 新的规范，这些东西都会是包裹在 easyspider里面的
        # 默认是存入MySQL，所以默认值为True 来让他存入
        if not _save_item.get("save_mysql", True) or not _save_item.get("easyspider", {}).get("save_mysql", True):
            return item
        _save_item.pop("crawled_urls_path")
        # _save_item.pop("crawled_url")
        _save_item.pop("spider")
        _save_item.pop("crawled_server")
        _save_item.pop("crawled_time")
        _save_item.pop("crawled_url")
        # 用什么校验，来防止重复插入
        hash_check_item = _save_item.pop("hash_check_item", None)

        # # 兼容以前老代码的写法
        # table = None
        # db = None
        # hash_check_item = None
        # unique_key = None
        # hash_check_enable = None
        if _save_item.get("easyspider"):
            easyspider_meta = _save_item.pop("easyspider")
        else:
            easyspider_meta = {}
        mysql_config = easyspider_meta.get("mysql_config") or {}

        table = mysql_config.get("table")
        db = mysql_config.get("db")
        # 兼容以前的老版写法，以前的写法是，hash_check直接写在最外面
        # 现在是如果没有发现以前的老版本写法的话，那么就使用新版本的写法
        if not hash_check_item:
            hash_check_item = mysql_config.get("hash_check_item")
        unique_key = mysql_config.get("unique_key")
        hash_check_enable = mysql_config.get("hash_check_enable", True)
        # 最后

        """下面是为了解决 mysql has gone away 错误
        尝试了好多种办法，想从 DBService 里面解决，但是DBService 解决的话，不阻塞当前任务去循环，就只能开线程
        开线程以后，错误就没办法汇报上来了

        所以这有在这里 sleep 来操作，来尝试
        因为都欧式 deferToThread，所以来使用sleep 也就没关系
        """
        error_flag = False
        error_limit = 3
        error_count = 1
        while error_count <= error_limit:
            try:
                _copy_save_item = copy.deepcopy(_save_item)
                self._insert_or_update(_copy_save_item, spider, table, db, hash_check_item, unique_key, hash_check_enable)
                error_flag = False
                break
            except Exception:
                error_flag = True
                logger.exception(u"第[%s / %s]操作数据库失败，下次再尝试" % (error_count, error_limit))
                time.sleep(2)

            error_count += 1

        # 如果最后一次还是错了，那么再来一次，还是不行，就算了
        if error_flag:
            self._insert_or_update(_save_item, spider, table, db, hash_check_item, unique_key, hash_check_enable)

        return item
    """
    # 以前的老版本，现在更新，能在没有 hash_check 的情况下，也能 insert_or_update
    def _insert_or_update(self, item, spider, table=None, db=None, hash_check_item=None):
        if not hash_check_item:
            _hash_check_item = item
        else:
            _hash_check_item = {}
            for check_item in hash_check_item:
                _hash_check_item[check_item] = item.get(check_item)
        hash_code = md5_from_dict(_hash_check_item)

        if not table:
            # current_table = "%s.%s" % (self.mysql_db, self.mysql_table)
            current_table = self.mysql_table
        else:
            current_table = table
        if not db:
            current_db = self.mysql_db
        else:
            current_db = db
        hashcode_list = self.server.query('select id from %s.%s where hash_check="%s";' % (
            current_db, current_table, hash_code))
        if hashcode_list:
            record_id = hashcode_list[0].get("id")
            item["last_checktime"] = current_time()
            update_sql = self.server.update_sql_from_map(
                current_table, {"id": record_id}, item, current_db).replace("%", "%%")
            logger.debug(
                "already have record, update last_checktime, running sql is %s" % update_sql)
            self.server.execute(update_sql)
        else:
            item["hash_check"] = hash_code
            item["last_checktime"] = current_time()
            sql = self.server.join_sql_from_map(
                current_table, item, current_db).replace("%", "%%")
            logger.debug("find a new record, insert sql is %s" % sql)
            self.server.execute(sql)
    """

    def _insert_or_update(self, item, spider, table=None, db=None, hash_check_item=None, unique_key="id", hash_check_enable=True):

        # 适应可以指定表的情况
        # -------------------------------------------------------------
        if not table:
            current_table = self.mysql_table
        else:
            current_table = table

        if not db:
            current_db = self.mysql_db
        else:
            current_db = db

        if not hash_check_item:
            # 注意，这里没有copy 因为_hash_check_item不会去改变源数据
            _hash_check_item = item
        else:
            # 有hash_check_item，重新装载
            _hash_check_item = {}
            for check_item in hash_check_item:
                _hash_check_item[check_item] = item.get(check_item)

        # 默认以 id 作为unique key
        if not unique_key:
            unique_key = "id"

        if hash_check_enable:
            # 启用hash_check_enable的话，简单直观就是去检索 hash_code
            hash_code = md5_from_dict(_hash_check_item)
            query_map = 'hash_check="%s"' % hash_code
        else:
            # 不启用，只好一个个的 and 下去
            query_map = MysqlService.join_query_map(_hash_check_item)

        check_sql = "select %s from %s.%s where %s;" % (unique_key, current_db, current_table, query_map)
        # 最终生成的查询，如果是hash_check 形式的话，那么就是原来的
        # hashcode_list = self.server.query('select id from %s.%s where hash_check="%s";' % (current_db, current_table, hash_code))
        # 如果不是 hash_check形式的话，那么就是组合的 key=value
        check_result = self.server.query(check_sql)
        if check_result:
            # 找到记录，那么后续的操作就是update更新
            unique_key_item = check_result[0].get(unique_key)
            if hash_check_enable:
                item["last_checktime"] = current_time()
            update_sql = self.server.update_sql_from_map(
                current_table, {unique_key: unique_key_item}, item, current_db).replace("%", "%%")
            logger.debug(
                "already have record, update last_checktime, running sql is %s" % update_sql)
            self.server.execute(update_sql)
        else:
            # 找不到记录，那么就是直接插入
            if hash_check_enable:
                item["hash_check"] = hash_code
                item["last_checktime"] = current_time()
            sql = self.server.join_sql_from_map(
                current_table, item, current_db).replace("%", "%%")
            logger.debug("find a new record, insert sql is %s" % sql)
            self.server.execute(sql)



# -*- coding: utf-8 -*-
# @Author: hang.zhang
# @Date:   2017-07-04 10:05:32
# @Last Modified by:   hang.zhang
# @Last Modified time: 2017-07-05 14:30:55
"""
    前面固定的模板，用于引用easyspider里面提供的数据库支持代码
    把spider和etl分开方便，整洁，好看
"""
import sys
reload(sys)
sys.setdefaultencoding("utf-8")
sys.path.append("../easyspider/utils")


def main():
    pass

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# @Author: hang.zhang
# @Date:   2017-07-19 15:35:14
# @Last Modified by:   hang.zhang
# @Last Modified time: 2017-07-19 19:14:25
from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler
import ConfigParser
import argparse
import json
import re
import os

class log_monitor_server(FileSystemEventHandler):


    def on_modified(self, events):
        modified_file = events.src_path
        if os.path.abspath(str(modified_file)) in self.get_ignore_file():
            print "SKIP, modified file %s in ignore file list" % modified_file
            return
        print "file %s modified" % modified_file



    def start_monitor_server(self):
        self.args = self.parse_args()
        self.cfg = ConfigParser.ConfigParser()
        observer = Observer()
        observer.schedule(self, path=self.args.d, recursive=self.args.r)
        observer.start()
        # 这个会导致cpu的使用激增
        while True:
            pass
        observer.join()

    def parse_args(self):
        parser = argparse.ArgumentParser(description="日志监控服务")
        parser.add_argument("-d", help="监控目录，默认为当前目录", default="./")
        parser.add_argument("-r", help="递归监控目录", default=True)
        parser.add_argument("-cfg_file", help="监控目录，默认为当前目录", default="log_monitor.cfg")
        args = parser.parse_args()
        return args

    def get_ignore_file(self):
        self.cfg.read(self.args.cfg_file)
        return [os.path.abspath(ignore_file) for ignore_file in self.cfg.options("ignore")]


if __name__ == '__main__':
    log_monitor_server().start_monitor_server()



# -*- coding: utf-8 -*-
# @Author: hang.zhang
# @Date:   2017-07-19 09:08:33
# @Last Modified by:   hang.zhang
# @Last Modified time: 2017-07-27 15:46:56
from base_etl import base_etl
from dbConn_for_etl import join_insert_sql_for_print
import logging
logger = logging.getLogger(__name__)

"""
python base_mongo_result_2_csv.py -mongo-collection noaaCrawler
"""
class base_mongo_result_2_csv(base_etl):

    def run(self):
        args = self.parse_cmd()

        logger.info("start connect mongo & get result")
        mongo_result = self.get_mongo_all_result(
            args.mongo_url, args.mongo_db_name, args.mongo_collection)

        # 一般来说拿出来mongo里面的内容要个性化解析之后，才能存到redis里面作为起始链接
        logger.info(
            "get mongo data successed, get %d mongo result, now start clean mongo result" % len(mongo_result))
        clean_result = self.parse_mongo_result(mongo_result)

        logger.info(
            "clean mongo result successed, now mongo result count is %d, now start convert mongo result -> sheet" % len(clean_result))
        sheet_result = [self._convert_mongo_2_sheet(result, args.interval, filter_callback=self.filter_sheet).copy() for result in clean_result]

        if not args.csv_file:
        	args.csv_file = "result_csv_%s.csv" % args.mongo_collection
        self.write_2_csv_file(sheet_result, args.csv_file, args.csv_split_symbol)

        logger.info("all successed !")

    # 这个函数子类可以覆盖，来达到个性化解析mongodb内容的效果
    # 默认情况是，是只提取mongo里面的url字段，并且不做处理直接插入
    # 一种虽然只需要url字段但是需要处理的情况是：url某个字段错了，需要批量替换
    def parse_mongo_result(self, mongo_result):
        # 都需要去除_id字段
        map(lambda result: result.pop("_id"), mongo_result)
        return mongo_result

    def write_2_csv_file(self, result_sheets, csv_file, csv_split_symbol):
        fields = set()
        for result in result_sheets:
            for k in result.keys():
                fields.add(k)
        fields = list(fields)
        with open(csv_file, "w") as f:
        	f.write("%s\n" % csv_split_symbol.join(fields))
        	# f.write("%s\n" % csv_split_symbol.join([ result.get(key,"None")  for key in fields  for result in result_sheets]))
        	for result in result_sheets:
        		f.write("%s\n" % csv_split_symbol.join([result.get(key, "None").replace("\n", "") for key in fields]))

    def filter_sheet(self, key_name):
    	# if key_name in ("detail", "bottom_table"):
    	# 	return False
    	return True
if __name__ == '__main__':
    base_mongo_result_2_csv().run()



#coding=utf-8

import logging
from logging.config import dictConfig

DEFAULT_LOGGER_CONFIG = {
    "version":1,
    "handlers":{
        "console":{
            "class":"logging.StreamHandler",
            "formatter":"default_formatter",
        },
        "file":{
            "class":"logging.FileHandler",
            "formatter":"default_formatter",
            "filename":"./log_base.log"
        }
    },
    "formatters":{
        "default_formatter":{
            "format":"%(asctime)s [%(name)s] %(levelname)s: %(message)s",
            "datefmt":"%Y-%m-%d %H:%M:%S"
        }
    },
    "root":{
        "level":"DEBUG",
        "handlers":["console","file"]

    }
}

def config_logger():
    dictConfig(DEFAULT_LOGGER_CONFIG)

config_logger()

logger = logging.getLogger(__name__)
logger.info("logger start")



# coding=utf-8

from easyspider.utils.userAgentList import UA_headers
import random


class userAgentMiddleware(object):

    def process_request(self, request, spider):
        this_time_ua = random.choice(UA_headers)
        if this_time_ua:
            request.headers.setdefault(
                "User-Agent", this_time_ua["User-Agent"])



# -*- coding: utf-8 -*-
# @Author: hang.zhang
# @Date:   2018-03-05 19:46:37
# @Last Modified by:   hang.zhang
# @Last Modified time: 2018-03-05 19:47:56

from scrapy.pipelines import ItemPipelineManager


class easyItemPipelineManager(ItemPipelineManager):

    def process_item(self, item, spider, response):
        return self._process_chain('process_item', item, spider, response)



# -*- coding: utf-8 -*-
# @Author: hang.zhang
# @Date:   2017-08-02 16:45:33
# @Last Modified by:   hang.zhang
# @Last Modified time: 2017-08-03 11:19:30

import easyspider
from os.path import join, exists, abspath
from scrapy.commands.startproject import Command
from shutil import ignore_patterns, move, copy2, copystat

class easyCommand(Command):

    def run(self, args, opts):
        if len(args) not in (1, 2):
            raise UsageError()

        project_name = args[0]
        project_dir = args[0]

        if len(args) == 2:
            project_dir = args[1]

        if exists(join(project_dir, 'scrapy.cfg')):
            self.exitcode = 1
            print('Error: scrapy.cfg already exists in %s' % abspath(project_dir))
            return

        if not self._is_valid_name(project_name):
            self.exitcode = 1
            return

        self._copytree(self.templates_dir, abspath(project_dir))
        """
        # 只是完整的复制过来，创建修改时间和文件夹的名字都没有变
        # 不方便记录新增时间
        """
        # print join(project_dir, 'crawler')
        # print join(project_dir, project_name)
        # 不需要把scrapy里面的文件夹名字也变成project_name, 直接用crawler挺好的
        # 只要最外层是project_name 就好了
        # move(join(project_dir, 'crawler'), join(project_dir, project_name))
        
        # TODO: 创建时间修改时间问题..希望能是最新的



    @property
    def templates_dir(self):
        _templates_base_dir = join(easyspider.__path__[0], 'template_dir')
        return join(_templates_base_dir)


# coding=utf-8
from easyspider.spiders.easyCrawlSpider import easyCrawlSpider
import urllib
import re
import json
import copy


class baiduSpider(easyCrawlSpider):

    name = 'baiduSpider'

    def start_requests(self):
        """you can add to redis queue or directly yield scrapy.Request to launch request(s)
        del start queue in redis: del baiduSpider:start_urls baiduSpider:dupefilter
        """
        template_start_urls_type = {
            "url": "",                  # Requirement
            "method": "GET",            # optional
            "headers": "",              # optional
            "post_data": {},            # optional, dict or urlencode str
            "cookies": "",              # optional
            "easyspider": {             # optional, easyspider's remark info
                # optional, to mark if this start_url is reloaded from retry
                # middleware
                "from_retry": 0,
                # optional, to mark request's original url, it is useful when
                # multi redirect happend.
                "source_start_url": "",
                "start_url_remark": "easyspider's test module"  # request's remark message
            }
        }
        # add request to redis task queue, search in baidu 'A' -> 'Z'
        search_template_url = "https://www.baidu.com/s?wd=%s"
        search_keyword_alphabet = [chr(i)
                                   for i in xrange(ord("A"), ord("Z") + 1)]

        for keyword in search_keyword_alphabet:
            # attention, use deep copy
            start_info = copy.deepcopy(template_start_urls_type)
            start_info.update({
                "url": search_template_url % keyword
            })
            self.server.rpush("%s:start_urls" % (self.name),
                              json.dumps(start_info, ensure_ascii=False))

    def parse(self, response):

        if not hasattr(self, "rex_extract_keyword"):
            self.rex_extract_keyword = re.compile("(wd=(.*?)&|wd=(.*?)$)")

        search_url = str(response.url)
        search_keyword = urllib.unquote((self.rex_extract_keyword.findall(
            search_url)[0][1] or self.rex_extract_keyword.findall(search_url)[0][2]))
        page = ";".join(response.xpath(
            "//span[@class='fk fk_cur']/../span[2]/text()").extract())

        for div in response.xpath("//div[@class='result c-container ']"):
            title = ";".join(div.xpath("h3/a/text()").extract())
            href = ";".join(div.xpath("h3/a/@href").extract())
            desc = ";".join(div.xpath("div[1]//text()").extract())
            yield {
                "search_url": search_url,
                "search_keyword": search_keyword,
                "page": page,
                "title": title,
                "href": href,
                "desc": desc,
            }



# -*- coding: utf-8 -*-
# @Author: hang.zhang
# @Date:   2017-08-04 09:19:20
# @Last Modified by:   hang.zhang
# @Last Modified time: 2017-08-06 18:57:02

"""DB tools

"""

import json
import logging
import threading
from collections import Iterable
from importlib import import_module
# from logger_base import *

logger = logging.getLogger(__name__)


class DBService(threading.local):

    def __getattr__(self, key):
        logger.debug(
            "menthod or attribute %s does no exists, now will be interperted call as <self.server.%s>" % (key, key))
        # return eval("super(DBService, self).server.%s" % key)
        # return eval("super(DBService, self).__getattr__('server').%s" % key)
        return eval("super(DBService, self).__getattribute__('server').%s" % key)


class RedisService(DBService):
    """this class is an improvement to the native python redis class.
    it has overwirte some function to make you use redis more convenient.

    it just overwrite the methods in native redis(but did not inherit it),
    so the methods name are keep the same, that means you don't need to adapt
    new api.
    one of the most amazing thing is, this class use `__getattr__` to bind
    unoverwritten's native redis method on  itself,

    when the method you called didn't exist
    in this class(but exists in native redis), you can also call it because at
    this time, you call will be interperted as call a Redis object.


    for example
        1. it has overwrite redis `rpush`, in this rpush, it
        will auto check the argument is list or string and if argument
        is provide as list type(iterable type), it will use redis pipe
        to transfer the command and just submit once.

        2. if the method is not overwritetn, you can also use it because
        when the method is not exists, your call will be interperted as
        call Redis object.
    """

    def __init__(self, redis_url):
        self.redis = import_module("redis")
        pool = self.redis.ConnectionPool.from_url(redis_url)
        self.server = self.redis.StrictRedis(connection_pool=pool)

    def join_pipe_commands(self, key, vals, op):
        """if given a large list or set to insert, it will auto call _pipe"""
        _pipe = self.server.pipeline()
        """warning: you must be carefull to the order !!! isinstance str must
        in the top level`if`, otherwise, insert `hello` will be interperted
        as insert [`h`,`e`,`l`,`l`,`o`]
        """
        if isinstance(vals, str) or isinstance(vals, unicode):
            eval("_pipe.%s(key, vals)" % op)
        elif isinstance(vals, Iterable):
            logger.debug("start join redis pipe...")
            for val in vals:
                eval(
                    "_pipe.%s(key, isinstance(val, dict) and json.dumps(val, ensure_ascii=False) or val)" % op)
        else:
            logger.error(
                "rpush vals %s's type is no avaliabled, type is %s" % (vals, type(vals)))
            return
        logger.debug("join redis pipe complete...start transfer to server")
        # if the return value is None indicate may occur some problems
        return _pipe.execute()

    def rpush(self, key, vals):
        return self.join_pipe_commands(key, vals, "rpush")

    def sadd(self, key, vals):
        return self.join_pipe_commands(key, vals, "sadd")


class MongoService(DBService):
    """all function args isn't bind at self, so you can directily use like this
    when you first use mongo connection ,it will bind with instance, and next time
    wher you use it, it will auto check if the instnace bind address has changed

    >>>server = MongoService()
    >>>server.find_all("mongodb://localhost:27017", "test")
    >>>server.find_all("mongodb://114.114.114.114:27017", "test")

    """

    def __init__(self, url=None, db=None):
        self.pymongo = import_module("pymongo")
        self.errors = import_module("pymongo.errors")
        if url:
            self.get_mongo_connection(url, db)

    def get_mongo_connection(self, url, db):
        # self.mongo_url & self.mongo_db to let you know your mongo location
        self.current_mongo_url = url
        self.current_mongo_db = db

        # MongoClient will auto call pymongo.uri_parser.parse_uri to check mongo url if valid
        # if invalid, function will raise InvalidURI Exception
        try:
            self.server = self.pymongo.MongoClient(url)
        except self.errors.InvalidURI:
            logger.exception("mongourl <%s> is invalid..." % url)
            exit(-1)

        self.select_db(db)

    def select_db(self, db=None):
        if db:
            self.db = self.server[db]
            self.current_mongo_db = db
        else:
            # db can provide in mongo url, if mongo url have not provide
            # database, get_default_database will raise ConfigurationError
            try:
                self.db = self.server.get_default_database()
                self.current_mongo_db = db
            except self.errors.ConfigurationError:
                self.db = None

    def find_one(self, collection, query_sql):
        """actually the function will call function find_limit and give the limit argument as 1"""
        return self.find_limit(collection, query_sql, 1)

    def find_all(self, collection, query_sql):
        """alike the function find_one, actually call find_limit"""
        return self.find_limit(collection, query_sql, 0)

    def find_limit(self, collection, query_sql, limit):
        """alike find_one, actually call function find_limit_and_skip"""
        return self.find_limit_and_skip(collection, query_sql, limit, 0)

    def find_limit_and_skip(self, collection, query_sql, limit, skip):
        """alike find_limit, actually call function find"""
        return self.find(collection, query_sql, limit, skip)

    def find(self, collection, query_sql, limit_count, skip_count=0, fetch_rate=5000):
        """ the <root> function when you run 'query' commands will called.
        no matter what query commands is(find_one or find_all or find_limit, etc.), actually it just the different kind
        of arguments to called this root function `find`.

        find is from pymongo.collection.find, ref: https://api.mongodb.com/python/current/api/pymongo/collection.html
        but i wanner explain what the argument `fetch_rate` is:

        when your query return a large numbers of result(like use find_all, to extract all result in a collection),
        you may wait until query done and fetch completed for a long time. during this period, you don't know
        how many querys has been done or even if query still alive(sometime the query will crashed).
        fetch_rate is the argument that control how many results to fetch in one time(default to 5k), and with the fetch_rate,
        you can directily know the query rate.
        """

        # TODO: check server or db if valid
        #
        # print collection, query_sql, limit_count, skip_count
        result_count = self.db[collection].find(
            query_sql, limit=limit_count, skip=skip_count).count(with_limit_and_skip=True)
        rate, result_list = 0, []
        while rate < result_count:
            # _id will cause TypeError: ObjectId('5982997934cbbaf8d033f86c') is not JSON serializable
            # and the dict extract from mongo is unicode, for example:{u'crawled_server': u'localhost'}
            # eval and json.dumps are used to convert to truly dict type
            result_list.extend([i.pop("_id") and eval(json.dumps(i, ensure_ascii=False)) for i in self.db[collection].find(
                query_sql, limit=fetch_rate if limit_count > fetch_rate else limit_count, skip=rate if rate > skip_count else skip_count)])
            rate += fetch_rate
            logger.debug("fetch rate: [%d / %d]" % (rate, result_count))
        return result_list

    def insert(self, collection, document):
        """auto check insert many or insert_one"""
        __insert = self.db[collection].insert_many(
            [document] if type(document) not in (list, set, tuple) else document)
        return len(__insert.inserted_ids)

    def replace_one(self, collection, query_sql, document):
        """mongo save(replace), has not save multi argument, replace or save can only replace one"""
        # return self.replace(collection, query_sql, document, multi=False)
        __replace = self.db[collection].replace_one(query_sql, document)
        return __replace.modified_count

    def delete_one(self, collection, query_sql):
        return self.delete(collection, query_sql)

    def delete_all(self, collection, query_sql):
        return self.delete_all(collection, query_sql, True)

    def delete(self, collection, query_sql, multi=False):
        """merge delete_one and delete_many by use argument multi"""
        __delete = self.db[collection].delete_one(query_sql) if not multi else self.db[
            collection].delete_many(query_sql)
        return __delete.deleted_count

    def list_all_collections(self, db=None):
        if not db or self.current_mongo_db == db:
            return self.convert_unciode_result(self.db.collection_names())
        else:
            # if db not match, change db & recall againe
            self.select_db(db)
            return self.list_all_collections(db)

    def list_all_dbs(self):
        """ convert [u"admin", u"local"] -> ["admin", "local"]"""
        return self.convert_unciode_result(self.server.database_names())

    def collection_result_count(self, collection):
        return self.db[collection].count()

    def convert_unciode_result(self, result):
        """ convert [u"admin", u"local"] -> ["admin", "local"]
        or {u"中文": u"not bad"} -> {"中文": "not bad"}
        because somethings bas will occur when the result is not in english
        """
        return eval(json.dumps(result, ensure_ascii=False))


class MysqlService(DBService):

    @classmethod
    def join_sql_from_map(cls, insert_table, args_map):
        """ magic method, surprise method, a very helpful method to
        allow you generate insert sql script from a dict.

        e.g:
        >>>MysqlService.join_sql_from_map("test_table", {"first_name": "zhang", "last_name": "yiTian"})
        >>>'insert into test_table(`first_name`,`last_name`) value("zhang","yiTian");'

        >>>MysqlService.join_sql_from_map("test_table", {"first_name": "zhang", "last_name": 'yi"Tian'})
        >>>'insert into test_table(`first_name`,`last_name`) value("zhang","yi\"Tian");'

        with the help of this method, you no longer warry about how to generate a sql script.
        """

        sql_template = "insert into %s(%s) value(%s);"
        # dict has no order, so you should get all dict content in one time.
        dict_list = args_map.items()

        fields = ",".join(map(lambda x: "`%s`" % x[0], dict_list))
        value = ",".join(map(lambda x: '"%s"' %
                             x[1].replace('"', '\\"'), dict_list))
        return sql_template % (insert_table, fields, value)

    def __init__(self):
        pass


def test_mongo_service():
    server = MongoService("mongodb://localhost:27017/easyspider")
    print "all mongo dbs are %s" % server.list_all_dbs()
    print "db easyspider's collections are %s" % server.list_all_collections()

    # test call not exist method in MongoService, pay attention to this result,
    # original method collection_names will return like [u"admin"] not
    # ["admin"]
    print "db easyspider's collections are %s" % server.easyspider.collection_names()

    # test change db
    print "db admin's collections are %s" % server.list_all_collections("admin")
    print "select db easyspider again", server.select_db("easyspider")

    # test insert one
    insert_result = {"test": "hello"}
    print "insert %s " % insert_result, server.insert("test_collection", insert_result)

    # test insert many
    insert_result = [{"info2": ["1", "中文", "english"]},
                     {"info2": ["2", "中文", "english"]}]
    print "insert %s " % insert_result, server.insert("test_collection", insert_result)

    # test insert many tyep is tuple
    insert_result = ({"info2": ["1", "中文", "english"]}, {
                     "info2": ["2", "中文", "english"]})
    print "insert %s " % str(insert_result), server.insert("test_collection", insert_result)

    # test find method, find_one
    print "find one in test_collection ", server.find_one("test_collection", {})

    # test find all
    print "find all in test_collection ", server.find_all("test_collection", {})

    # test find last result
    collection_result_count = server.collection_result_count("test_collection")
    print "test_collection have %s result" % collection_result_count
    print "test_collection last result is %s" % server.find_limit_and_skip("test_collection", {}, 1, collection_result_count - 1)

    # test delete one
    print "test del one %s" % server.delete("test_collection", {})
    # count now result
    collection_result_count = server.collection_result_count("test_collection")
    print "test_collection have %s result" % collection_result_count
    # test delete all
    print "test del all %s" % server.delete("test_collection", {}, multi=True)


def test_redis_service():
    server = RedisService("redis://127.0.0.1")
    print server.server.ping()
    # test call not exists method in RedisService
    print server.ping()

    # test rpush list
    print server.rpush("test", [i for i in range(10)])
    print server.rpush("test2", ["hello", {"zhang": "yiTian"}, 3])

    # test rpush str
    print server.sadd("test3", "ni hao")
    print server.sadd("test4", ["你好", "China"])

    # test call not exists method in RedisService
    print server.hset("test5", "zhang", {"Tian": 1})

    # test if need to convert unicode
    print server.hset("test5", "中国", "no bad")
    print server.hgetall("test5")
    print server.hgetall("test5").keys()[0], server.hgetall("test5").values()[0]


def test_mysql_service():
    print MysqlService.join_sql_from_map("test_table", {"first_name": "zhang", "last_name": "yiTian"})
    print MysqlService.join_sql_from_map("test_table", {"first_name": "zhang", "last_name": 'yi"Tian'})


def main():
    # test_mongo_service()
    # test_redis_service()
    test_mysql_service()

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# @Author: hang.zhang
# @Email:  hhczy1003@163.com
# @Date:   2017-08-01 20:37:27
# @Last Modified by:   hang.zhang
# @Last Modified time: 2017-08-01 23:05:11

from setuptools import setup

setup(
    name="easyspider",
    version="1.8.1",
    author="yiTian.zhang",
    author_email="hhczy1003@163.com",
    packages=["easyspider"],
    include_package_data=True,
    zip_safe=False,
    entry_points={
        "console_scripts": ["easyspider = easyspider.core.cmdline:execute"]
    },
    install_requires=[
        "xmltodict",
        "requests",
        "BeautifulSoup4",
        "Django",
        "scrapy_redis",
        "pymysql",
        "dbutils",
    ]
)



# -*- coding: utf-8 -*-
# @Author: hang.zhang
# @Date:   2017-08-02 16:45:33
# @Last Modified by:   hang.zhang
# @Last Modified time: 2017-08-02 17:03:17

from scrapy.commands.crawl import Command


class easyCommand(Command):
    pass




# coding=utf-8

UA_headers = [  # 浏览器头信息 (PC端)
    # 我自己浏览器标志
    {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/44.0.2403.130 Safari/537.36'},
    # safari 5.1 – MAC
    {'User-Agent': 'Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50'},
    # safari 5.1 – Windows
    {'User-Agent': 'Mozilla/5.0 (Windows; U; Windows NT 6.1; en-us) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50'},
    # Firefox 4.0.1 – MAC
    {'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:2.0.1) Gecko/20100101 Firefox/4.0.1'},
    # Firefox 4.0.1 – Windows
    # {'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; rv:2.0.1) Gecko/20100101 Firefox/4.0.1'},
    # Opera 11.11 – MAC
    # {'User-Agent': 'Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; en) Presto/2.8.131 Version/11.11'},
    # Opera 11.11 – Windows
    # {'User-Agent': 'Opera/9.80 (Windows NT 6.1; U; en) Presto/2.8.131 Version/11.11'},
    {'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.90 Safari/537.36 OPR/47.0.2631.71'},
    # Chrome 17.0 – MAC
    {'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11'},
    # 傲游（Maxthon）
    # # 在某些服务器上，已经不能使用，会报浏览日过老的错误。
    # {'User-Agent': 'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Maxthon 2.0)'},
    # 腾讯TT
    # # 在某些服务器上，已经不能使用，会报浏览日过老的错误。
    # {'User-Agent': 'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; TencentTraveler 4.0)'},
    # 世界之窗（The World） 3.x
    # # 在某些服务器上，已经不能使用，会报浏览日过老的错误。
    # {'User-Agent': 'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; The World)'},
    # IE 9.0
    # # 在某些服务器上，已经不能使用，会报浏览日过老的错误。
    # {'User-Agent': 'Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0;'},
    # IE 8.0
    # 在某些服务器上，已经不能使用，会报浏览日过老的错误。
    # {'User-Agent': 'Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0)'}
]

r"""

opera ua header is error... must start with Mozilla*
Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.90 Safari/537.36 OPR/47.0.2631.71






http://www.zydmall.com/detail/good_2919.html


2017-09-07 10:44:58 [zydDetail] INFO: http://www.zydmall.com/ashx/detail_product
s.ashx is recorded, because Spider error processing <POST http://www.zydmall.com
/ashx/detail_products.ashx> (referer: http://www.zydmall.com/detail/good_3240.ht
ml): info (<class 'scrapy.spidermiddlewares.httperror.HttpError'>, HttpError('Ig
noring non-200 response',), <traceback object at 0x00000000058CBE08>) happended,
 following are detail info:
            response url -> http://www.zydmall.com/ashx/detail_products.ashx,
            status code -> 404,
            request url -> http://www.zydmall.com/ashx/detail_products.ashx,
            original request url -> http://www.zydmall.com/ashx/detail_products.
ashx

            request headers -> {'Accept-Language': ['en'], 'Accept-Encoding': ['
gzip,deflate'], 'Accept': ['text/html,application/xhtml+xml,application/xml;q=0.
9,*/*;q=0.8'], 'User-Agent': ['Opera/9.80 (Windows NT 6.1; U; en) Presto/2.8.131
 Version/11.11'], 'Referer': ['http://www.zydmall.com/detail/good_3240.html'], '
Content-Type': ['application/x-www-form-urlencoded; charset=UTF-8']},
            request body -> {'pid': '255072', 'goods_id': '3240'},

            request callback -> self.parse,

            response body -> <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict
//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=gb2312"/>
<title>404 - 找不到文件或目录。</title>
<style type="text/css">
<!--
body{margin:0;font-size:.7em;font-family:Verdana, Arial, Helvetica, sans-serif;b
ackground:#EEEEEE;}
fieldset{padding:0 15px 10px 15px;}
h1{font-size:2.4em;margin:0;color:#FFF;}
h2{font-size:1.7em;margin:0;color:#CC0000;}
h3{font-size:1.2em;margin:10px 0 0 0;color:#000000;}
#header{width:96%;margin:0 0 0 0;padding:6px 2% 6px 2%;font-family:"trebuchet MS
", Verdana, sans-serif;color:#FFF;
background-color:#555555;}
#content{margin:0 0 0 2%;position:relative;}
.content-container{background:#FFF;width:96%;margin-top:8px;padding:10px;positio
n:relative;}
-->
</style>
</head>
<body>
<div id="header"><h1>服务器错误</h1></div>
<div id="content">
 <div class="content-container"><fieldset>
  <h2>404 - 找不到文件或目录。</h2>
  <h3>您要查找的资源可能已被删除，已更改名称或者暂时不可用。</h3>
 </fieldset></div>
</div>
</body>
</html>

2017-09-07 10:44:59 [zydDetail] INFO: reput crawl task into start url, detail in
fo are {u'body': {'pid': '255072', 'goods_id': '3240'}, 'cookies': None, u'heade
rs': {'Accept-Language': ['en'], 'Accept-Encoding': ['gzip,deflate'], 'Accept':
['text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8'], 'User-Agent
': ['Opera/9.80 (Windows NT 6.1; U; en) Presto/2.8.131 Version/11.11'], 'Referer
': ['http://www.zydmall.com/detail/good_3240.html'], 'Content-Type': ['applicati
on/x-www-form-urlencoded; charset=UTF-8']}, u'url': u'http://www.zydmall.com/ash
x/detail_products.ashx', u'dont_filter': True, u'callback': 'self.parse', 'meta'
: {u'last_page_result': {u'category_channel': u'\u9996\u9875>\u4f4e\u538b\u914d\
u7535>\u5851\u58f3\u65ad\u8def\u5668>\u5851\u58f3\u914d\u7535\u4fdd\u62a4>\u5929
\u6b63\u7535\u6c14 THM1-250A 50KA \u56fa\u5b9a\u5f0f 3P \u5851\u58f3\u914d\u7535
\u4fdd\u62a4', u'pid': u'255072', u'main_panel': {u'\u6298\u6263\u4ef7\uff1a': u
'\xa5;780.00', u'\u9762\u4ef7\uff1a': u'\xa5780.00', u'\u7cfb\u5217\uff1a': u'TH
M1', u'\u54c1\u724c\uff1a': u'\u5929\u6b63\u7535\u6c14'}, u'goods_id': u'3240',
u'th_header': [u'\u8ba2\u8d27\u53f7', u'\u4ea7\u54c1\u578b\u53f7', u'\u9762\u4ef
7', u'\u6298\u6263\u4ef7', u'\u5e93\u5b58', u'\u8d27\u671f', u'\u6570\u91cf', u'
\u91cd\u91cf(g)', u'\u5355\u4f4d', u'\u58f3\u67b6\u7535\u6d41', u'\u5206\u65ad\u
80fd\u529b', u'\u8131\u6263\u5f62\u5f0f', u'\u8131\u6263\u5355\u5143', u'\u8131\
u6263\u5668\u989d\u5b9a\u7535\u6d41', u'\u6781\u6570', u'\u5b89\u88c5\u65b9\u5f0
f', u'\u63a5\u7ebf\u65b9\u5f0f', u'\u64cd\u4f5c\u65b9\u5f0f', u'\u4fdd\u62a4\u52
9f\u80fd', u'\u989d\u5b9a\u7535\u538b', u'\u9644\u4ef6']}, u'download_timeout':
7.0, u'depth': 1, u'download_latency': 0.06199979782104492, u'download_slot': u'
www.zydmall.com', u'easyspider': {u'from_retry': 1, u'remark': None, u'source_st
art_url': u'http://www.zydmall.com/ashx/detail_products.ashx'}}, u'method': 'POS
T'}
2017-09-07 10:44:59 [scrapy.extensions.logstats] INFO: Crawled 201 pages (at 60
pages/min), scraped 0 items (at 0 items/min)
















curl -vXPOST http://www.zydmall.com/ashx/detail_products.ashx \
-H "Content-Type': application/x-www-form-urlencoded; charset=UTF-8" \
-H "Referer: http://www.zydmall.com/detail/good_2919.html" \
-H "X-Requested-With: XMLHttpRequest" \
-H "User-Agent: Opera/9.80 (Windows NT 6.1; U; en) Presto/2.8.131 Version/11.11" \
-d "goods_id=2919&pid=229752" | iconv -f gbk

curl -vXPOST http://www.zydmall.com/ashx/detail_products.ashx \
-H "Content-Type': application/x-www-form-urlencoded; charset=UTF-8" \
-H "Referer: http://www.zydmall.com/detail/good_2919.html" \
-H "X-Requested-With: XMLHttpRequest" \
-H "User-Agent: Mozilla/5.0 (Windows NT 6.2; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.113 Safari/537.36" \
-d "goods_id=2919&pid=229752"


curl -vXPOST http://www.zydmall.com/ashx/detail_products.ashx \
-H "Content-Type': application/x-www-form-urlencoded; charset=UTF-8" \
-H "Referer: http://www.zydmall.com/detail/good_2919.html" \
-H "X-Requested-With: XMLHttpRequest" \
-H "User-Agent: Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; en) Presto/2.8.131 Version/11.11" \
-d "goods_id=2919&pid=229752" | iconv -f gbk

"""


# -*- coding: utf-8 -*-
# @Author: hang.zhang
# @Date:   2017-07-18 17:41:28
# @Last Modified by:   hang.zhang
# @Last Modified time: 2017-07-19 10:37:57

from base_etl import base_etl
from dbConn_for_etl import join_insert_sql_for_print
import logging
logger = logging.getLogger(__name__)

"""
python base_mongo_result_2_mysql.py -mongo-collection next_tcichemicals
"""
class base_mongo_result_2_mysql(base_etl):

    def run(self):
        args = self.parse_cmd()

        logger.info("start connect mongo & get result")
        mongo_result = self.get_mongo_all_result(
            args.mongo_url, args.mongo_db_name, args.mongo_collection)

        # 一般来说拿出来mongo里面的内容要个性化解析之后，才能存到redis里面作为起始链接
        logger.info(
            "get mongo data successed, get %d mongo result, now start clean mongo result" % len(mongo_result))
        clean_result = self.parse_mongo_result(mongo_result)

        logger.info(
            "clean mongo result successed, now mongo result count is %d, now start convert mongo result -> sheet" % len(clean_result))
        sheet_result = [self._convert_mongo_2_sheet(result, args.interval, filter_callback=self.filter_sheet).copy() for result in clean_result]

        if not args.mysql_file:
            args.mysql_file = "result_sql_%s.sql" % args.mongo_collection

        self.write_2_mysql_file(args.mongo_collection, sheet_result, args.mysql_file)

        logger.info("all successed !")

    # 这个函数子类可以覆盖，来达到个性化解析mongodb内容的效果
    # 默认情况是，是只提取mongo里面的url字段，并且不做处理直接插入
    # 一种虽然只需要url字段但是需要处理的情况是：url某个字段错了，需要批量替换
    def parse_mongo_result(self, mongo_result):
            # 都需要去除_id字段
        map(lambda result: result.pop("_id"), mongo_result)
        return mongo_result

    def write_2_mysql_file(self, table, result_sheet, sql_file):
        create_table_sql = self.create_table_sql(table, result_sheet)
        content_sql = self.create_content_sql(table, result_sheet)
        with open(sql_file, "w") as f:
            f.write(create_table_sql)
            f.write("\n".join(content_sql))

    def create_table_sql(self, table, result_sheets):
        fields = set()
        for result in result_sheets:
            for k in result.keys():
                fields.add(k)
        fields = list(fields)
        # 开始构造建表sql
        sql_head = "create table %s(\n" % table
        sql_foot = "\n)engine=InnoDB charset utf8;\n"
        sql_content = ",\n".join(["`%s` text" % f for f in fields])
        return sql_head + sql_content + sql_foot

    def create_content_sql(self, table, result_sheets):
        content_sql = []
        for result in result_sheets:
            sql = join_insert_sql_for_print(table, result)
            content_sql.append(sql)
        return content_sql

    def filter_sheet(self, key):
        if key in (u"规格"):
            return False
        return True
if __name__ == '__main__':
    base_mongo_result_2_mysql().run()
